"""Copyright 2024 Everlasting Systems and Solutions LLC (www.myeverlasting.net).
All Rights Reserved.

No part of this software or any of its contents may be reproduced, copied, modified or adapted, without the prior written consent of the author, unless otherwise indicated for stand-alone materials.

For permission requests, write to the publisher at the email address below:
office@myeverlasting.net

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

"""
from pydantic import BaseModel,Field, AnyHttpUrl, model_validator,EmailStr
from typing import List, Optional
from typing_extensions import Self
from datetime import datetime
from espy_contact.util.enums import ResourceEnum, GradeLevel, Term
from espy_contact.util.enums import StatusEnum
from espy_contact.schema.schema import UserResponse


class Resource(BaseModel):
    """Type of resource can be Poll, Form Builder, Questionnaire, RichText, Video, Audio, File, Hyperlink."""

    id: Optional[int] = None
    title: str
    type: ResourceEnum
    lesson_id: int


class Lesson_note(BaseModel):
    id: Optional[int] = None
    title: str
    content: str
    lesson_id: Optional[int] = None


class Quiz(BaseModel):
    id: Optional[int] = None
    title: str
    question: str
    options: List[str]
    answer: str
    lesson_id: Optional[int] = None


class Lesson(BaseModel):  # defintion of biology, branches of biology
    id: Optional[int] = None
    title: str  # Intro to Biology
    quiz: Optional[Quiz] = None
    note: Optional[Lesson_note] = None
    assets: Optional[
        List[str]
    ] = []  # these are assets shared between users not same as resources
    resources: Optional[List[ResourceEnum]] = []
    topic_id: Optional[int] = None


class LessonDto(Lesson):
    class_id: Optional[int] = None


class Topic(BaseModel):  # Introduction to Biology
    id: Optional[int] = None
    title: str
    timestamp: Optional[datetime] = None
    subject_id: Optional[str] = None
    lessons: Optional[List[Lesson]] = []


class TopicDto(Topic):
    age: Optional[datetime] = datetime


class SubjectDto(BaseModel):
    id: Optional[int] = None
    title: str  # Biology
    class_id: Optional[int] = None  # Grade
    grade: Optional[GradeLevel] = None
    term: Term
    topics: Optional[List[Topic]] = None
    lesson_count: Optional[int] = 0

    @model_validator(mode="after")
    def validate_subject_grade(self: 'SubjectDto') -> Self:
        if self.grade is None and self.class_id is None:
            raise ValueError("Either 'grade' or 'class_id' must be provided.")
        return self
    


class ClassroomDto(BaseModel):
    id: Optional[int] = None
    title: str
    subjects: Optional[List[SubjectDto]] = None
    teacher: Optional[UserResponse] = None  # ManyToMany relationship with Teacher


class Review(BaseModel):
    id: Optional[int] = None
    title: str
    review: str
    rating: float
    reviewer: str
    created_at: Optional[datetime] = None
    subject: SubjectDto

class Holiday(BaseModel):
    id: Optional[int] = None
    name: str
    remarks: Optional[str] = None
    start_date: datetime
    end_date: datetime
    created_at: Optional[datetime] = None
    created_by: str

class SchoolTerm(BaseModel):
    id: Optional[int] = None
    term: Term
    academic_year: int
    start_date: datetime
    end_date: datetime
    remarks: Optional[str] = None
    created_at: Optional[datetime] = None
    created_by: str

class SchoolProfile(BaseModel):
    id: Optional[int] = None
    school_id: int
    logo: Optional[str] = None
    mission: Optional[str] = None
    vision: Optional[str] = None
    motto: Optional[str] = None
    banner: Optional[str] = None
    contact: Optional[str] = None
    website: Optional[AnyHttpUrl] = None
    social_media: Optional[str] = Field(None, description='comma separated strings')
    established: Optional[int] = None
    school_type: Optional[str] = None
    affiliation: Optional[str] = None
    board: Optional[str] = None
    facilities: Optional[str] = Field(None, description='comma separated strings')
    created_at: Optional[datetime] = None

class Extracurricular(BaseModel):
    id: Optional[int] = None
    title: str
    description: str
    start_date: datetime
    end_date: datetime
    venue: Optional[str] = None
    created_by: str
    status: StatusEnum = StatusEnum.NEW
    coordinator: Optional[str] = None
    images: Optional[str] = Field(None, description='comma separated strings')
    created_at: Optional[datetime] = None

class Asset(BaseModel):
    id: Optional[int] = None
    title: str
    description: str
    worth: Optional[str] = None
    type: Optional[str] = None
    url: Optional[AnyHttpUrl] = None
    tags: Optional[str] = Field(None, description='comma separated strings')
    images: Optional[str] = Field(None, description='comma separated strings')
    created_by: str
    status: StatusEnum = StatusEnum.NEW
    created_at: Optional[datetime] = None



class Vacancy(BaseModel):
    id: Optional[int] = None
    title: str
    description: str = Field(description='detailed job description')
    requirements: Optional[str] = Field(None, description='comma separated strings')
    salary: Optional[str] = Field(None, description='amount with currency and duration e.g $48,000 per annum')
    location: Optional[str] = None
    deadline: datetime = Field(description='date you will stop accepting applications')
    created_by: str
    status: StatusEnum = StatusEnum.NEW
    created_at: Optional[datetime] = None

class Attendance(BaseModel):
    id: Optional[int] = None
    user_id: int
    subject_id: Optional[int] = None
    classroom_id: Optional[int] = None
    is_present: bool
    created_at: Optional[datetime] = None
    updated_at: Optional[datetime] = None
    created_by: EmailStr
    remarks: Optional[str] = None
    


"""Copyright 2024 Everlasting Systems and Solutions LLC (www.myeverlasting.net).
All Rights Reserved.

No part of this software or any of its contents may be reproduced, copied, modified or adapted, without the prior written consent of the author, unless otherwise indicated for stand-alone materials.

For permission requests, write to the publisher at the email address below:
office@myeverlasting.net

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

"""
from typing import List,Optional
from datetime import datetime,date,timedelta
from decimal import Decimal
from pydantic import BaseModel,Field, EmailStr, ConfigDict
from espy_contact.schema.schema import AppuserDto,AddressDto
from espy_contact.util.enums import StatusEnum
class ProductBase(BaseModel):
    logo: str
    website: str
    ga_property_id: str
    socials: str
    cost: Decimal = Field(max_digits=10, decimal_places=2)
    expiry_date: Optional[datetime] = None
    note: str

class ProductCreate(ProductBase):
    pass
class Product(ProductBase):
    model_config = ConfigDict(from_attributes=True)
    id: int
    customer_id: int

class CustomerBase(BaseModel):
    pass

class CustomerCreate(CustomerBase):
    model_config = ConfigDict(from_attributes=True)
    business_name: str
    business_email: EmailStr
    contact_name: str
    contact_phone: str
    contact_email: EmailStr
    address: AddressDto

class Customer(CustomerBase):
    model_config = ConfigDict(from_attributes=True)
    id: int
    staff: List[AppuserDto] = []
    products: List[Product] = []

class Campaign(BaseModel):
    id: Optional[int] = None
    title: str
    goals: Optional[str] = None
    target_audience: Optional[str] = None
    target_city: Optional[str] = None
    target_age_group: Optional[str] = None
    budget: float
    channels: Optional[str] = None
    ad_copy: Optional[str] = None
    expiry_date: datetime
    created_on: Optional[datetime]
    updated_on: Optional[datetime]
    note: Optional[str] = None
    product_id: int
    status: StatusEnum

class GaProperty(BaseModel):
    id: Optional[int] = None
    product_id: str
    start_date: Optional[date] = (datetime.now() - timedelta(weeks=30)).date()
    end_date: Optional[date] = datetime.now().date()


"""Copyright 2024 Everlasting Systems and Solutions LLC (www.myeverlasting.net).
All Rights Reserved.

No part of this software or any of its contents may be reproduced, copied, modified or adapted, without the prior written consent of the author, unless otherwise indicated for stand-alone materials.

For permission requests, write to the publisher at the email address below:
office@myeverlasting.net

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

"""
from espy_contact.util.db import Base
from espy_contact.util.enums import StatusEnum
from espy_contact.model.models import Appuser
from datetime import datetime
from sqlalchemy import Column, DateTime, ForeignKey, Date, Float,Enum
from sqlalchemy.sql.sqltypes import Integer, String, Boolean
from sqlalchemy.orm import relationship
from sqlalchemy.sql import func


class Account(Base):
    __tablename__ = "accounts"

    id = Column(Integer, primary_key=True, index=True, autoincrement=True)
    bank = Column(String, nullable=False)  # Assuming NigerianBank is a string enum
    account_name = Column(String, nullable=False)
    account_number = Column(String, nullable=False)
    currency = Column(String, nullable=False)
    is_active = Column(Boolean, default=True)
    account_officer = Column(String)
    account_admin = Column(String)
    created_on = Column(DateTime, default=datetime)
    modified_on = Column(DateTime, onupdate=datetime)


class Fee(Base):
    __tablename__ = "fees"

    id = Column(Integer, primary_key=True, index=True, autoincrement=True)
    classroom_id = Column(Integer, ForeignKey("classrooms.id"))
    fee_name = Column(String, nullable=False)
    amount = Column(Float, nullable=False)
    due_date = Column(Date, nullable=False)
    start_date = Column(Date, nullable=False)
    status = Column(Enum(StatusEnum), default=StatusEnum.NEW)
    created_on = Column(DateTime(), server_default=func.now())
    modified_on = Column(DateTime(), onupdate=func.now())
    creator_id = Column(Integer, ForeignKey("appusers.id"))


    classroom = relationship("Classroom")
    creator = relationship("Appuser")
